# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by spirito GmbH, 2002       ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.4 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/01/23 11:13:33 $"

import sys
import string
import cStringIO
import htmlentitydefs
import re
import cgi

_htmlentitydefs = htmlentitydefs.entitydefs
_htmlentitydefs["nbsp"] = " "

def UnescapeEntity(ref):
    if ref.startswith("#"):
        ref = ref[1:]
        ref = string.lower(ref)
        if ref[0]=="x":
            c = string.atoi(ref[1:], 16)
        else:
            c = string.atoi(ref, 10)
        if 0 < c < 256:
            return chr(c)
    else:
        c = _htmlentitydefs.get(ref, "")
        if c and c[0]=='&' and len(c)>1:
            c = ""
        if c:
            return c
    return ""

def _Unescape(m):
    return UnescapeEntity(m.group(1))

def Unescape(s, nbsp=0):
    if nbsp:
        s = s.replace("&nbsp;", "\xa0")
    return rxhtmlentity.sub(_Unescape, s)

htmlscript = """
    <\s*(?P<script>script)(?P<scriptattrs>\s.*?)>
        (?P<scriptdata>.*?)
    </script>
    """

htmlstyle = """
    <\s*(?P<style>style)(?P<styleattrs>\s.*?)>
        (?P<styledata>.*?)
    </style>
    """

htmlentity = """
    &   (?P<entity>
            \#[a-z0-9][0-9]*
            |
            [a-z0-9]+
        )
    ;
    """

rxhtmlentity = re.compile(
    htmlentity, 
    re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

#~ htmltag = """
    #~ <   \s*
        #~ (?P<closetag>/)?
        #~ \s*
        #~ (?P<tag>[a-z][a-z0-9_:\-]*)
        #~ (?P<attrs>\s.*?)?
        #~ /?
        #~ \s*
        #~ >
    #~ """

htmlattrs = """
    (
        \s+
        (?P<name>[a-z][a-z0-9_:\-]*)
        (
            \s*=\s*
            (?P<value>
                ".*?"
                |
                '.*?'
                |
                \S*
             )
        )?
    )
    """

htmltag_new = ("""
    <   \s*
        (?P<closetag>/)?
        \s*
        (?P<tag>[a-z][a-z0-9_:\-]*)
        (?P<attrs>
    """ + htmlattrs + """*)?
        (?P<singletag>/)?
        \s*
        >
    """)

htmltag = ("""
    <   \s*
        (?P<closetag>/)?
        \s*
        (?P<tag>[a-z][a-z0-9_:\-]*)
        (?P<attrs>\s.*?)?
        (?P<singletag>/)?
        \s*
        >
    """)
htmlxml = """
    <\? \s* (?P<xml>xml) \s* .*? \?>
    """
htmldoctype = """
    <\! \s* (?P<doctype>doctype)\s* (?P<doctypeattrs>.*?) >
    """

htmlcomment = """
    <\!\-\- (?P<comment>.*?) \-\->
    """

rxhtmlelements = re.compile("""
    (
    """ + htmlentity + """
    |
    """ + htmlscript + """
    |
    """ + htmlstyle + """
    |
    """ + htmltag + """
    |
    """ + htmlxml + """
    |
    """ + htmldoctype + """
    |
    """ + htmlcomment + """
    )
    """, 
    re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

rxhtmlelementsnoentities = re.compile("""
    (
    """ + htmlscript + """
    |
    """ + htmlstyle + """
    |
    """ + htmltag + """
    |
    """ + htmlxml + """
    |
    """ + htmldoctype + """
    |
    """ + htmlcomment + """
    )
    """, 
    re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

rxhtmlattrs = re.compile(
    htmlattrs,
    re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

def collapse(s):
    return " ".join(str(s).split()).strip()

def contains(a, b):
    return a.find(b)!=-1

class HTMLParser:
    """
    More ore less the same as sgmllib.SGMLParser
    """

    def __init__(self):
        pass
    
    def __del__(self):
        re.purge()

    def _getattrs(self, data):
        if not data:
            return {}
        attr = {}
        for m in rxhtmlattrs.finditer(data):
            name = m.group("name").lower()
            value = m.group("value") or name
            if value[0] in ('"', "'"):
                value = value[1:-1]
            attr[name] = Unescape(value).strip()
        return attr

    def parse(self, text, 
        script_safe=1, 
        style_safe=1, 
        encoding="latin1", 
        encodingmode="ignore"):

        text = unicode(text, encoding, encodingmode).encode("latin1", encodingmode)        

        self._parser_start = 0
        self._parser_end = 0
        self._parser_text = text
        self._parser_pos = pos = 0

        if script_safe and style_safe:
            rx = rxhtmlelements
        else:
            s = "(" + htmlentity + "|"
            if script_safe:
                s += htmlscript + "|"
            if style_safe:
                s += htmlstyle + "|"
            s += (
                htmltag + "|" +
                htmlxml + "|" +
                htmldoctype + "|" +
                htmlcomment + ")")

            rx = re.compile(s, 
                re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

        for m in rx.finditer(text):

            # print m.groupdict()
            self._parser_start = m.start()
            self._parser_end = m.end()

            data = text[pos:m.start()]
            if data:
                self.handle_data(data)

            data = m.group()
            if data.startswith("<"):
                if m.group("tag") is not None:
                    if m.group("closetag") is not None:
                        self.handle_endtag(m.group("tag").lower())
                    else:
                        self.handle_starttag(m.group("tag").lower(), self._getattrs(m.group("attrs")))
                elif m.group("comment") is not None:
                    self.handle_comment(m.group("comment"))
                elif script_safe and m.group("script") is not None:
                    self.handle_script(self._getattrs(m.group("scriptattrs")), m.group("scriptdata"))
                elif style_safe and m.group("style") is not None:
                    self.handle_style(self._getattrs(m.group("styleattrs")), m.group("styledata"))
                elif m.group("doctype") is not None:
                    self.handle_doctype(m.group("doctypeattrs"))
            else:
                ent = m.group("entity")
                if ent.startswith("#"):
                    try:
                        self.handle_charref(int(ent[1:]))
                    except:
                        pass
                else:
                    self.handle_entityref(ent)

            self._parser_pos = pos = m.end()

        data = text[pos:]
        if data:
            self.handle_data(data)

    feed = parse

    def get_lineno(self):
        return string.count(self._parser_text[:self._parser_pos], "\n") + 1
    def get_start_pos(self):
        return self._parser_start
    def get_end_pos(self):
        return self._parser_end
    def get_source(self):
        return self._parser_text

    def handle_doctype(self, data): pass
    def handle_style(self, attr, data): pass
    def handle_script(self, attr, data): pass
    def handle_comment(self, data): pass
    def handle_data(self, data): pass
    def handle_starttag(self, tag, attr): pass
    def handle_endtag(self, tag): pass
    def handle_charref(self, ref): pass
    def handle_entityref(self, ref): pass

class XMLParser(HTMLParser):

    def _getattrs(self, data):
        if not data:
            return {}
        attr = {}
        for m in rxhtmlattrs.finditer(data):
            name = m.group("name")
            value = m.group("value") or name
            if value[0] in ('"', "'"):
                value = value[1:-1]
            attr[name] = Unescape(value)
            #print attr[name]
        return attr

    def parse(self, text):

        self._parser_text = text
        self._parser_pos = pos = 0

        s = ("(" +
            htmlentity + "|" +
            htmltag + "|" +
            htmlxml + "|" +
            htmldoctype + "|" +
            htmlcomment + ")")

        rx = re.compile(s, 
            re.MULTILINE|re.DOTALL|re.IGNORECASE|re.VERBOSE)

        for m in rx.finditer(text):

            self._parser_start = m.start()
            self._parser_end = m.end()

            data = text[pos:m.start()]
            if data:
                self.handle_data(data)

            data = m.group()
            if data.startswith("<"):
                if m.group("tag") is not None:
                    if m.group("closetag") is not None:
                        self.handle_endtag(m.group("tag"))
                    else:
                        if m.group("singletag") is not None:
                            self.handle_tag(m.group("tag"), self._getattrs(m.group("attrs")))
                        else:
                            self.handle_starttag(m.group("tag"), self._getattrs(m.group("attrs")))
                elif m.group("comment") is not None:
                    self.handle_comment(m.group("comment"))
                elif m.group("doctype") is not None:
                    self.handle_doctype(m.group("doctypeattrs"))
            else:
                ent = m.group("entity")
                if ent.startswith("#"):
                    try:
                        self.handle_charref(int(ent[1:]))
                    except:
                        pass
                else:
                    self.handle_entityref(ent)

            self._parser_pos = pos = m.end()

        del rx
        
        data = text[pos:]
        if data:
            self.handle_data(data)

    feed = parse

    def handle_tag(self, tag, attr): 
        pass

def start_tag(tag, attr=None, escape=1):
    a = ""
    if attr:
        for k, v in attr.items():
            a += " " + k
            if v is not None:
                if escape:
                    a += "=\"" + cgi.escape(v,1) + "\""
                else:
                    a += "=\"" + str(v) + "\""
    return "<%s%s>" % (tag, a)

def end_tag(tag):
    return "</%s>" % (tag)

class HTMLWriter(HTMLParser):

    noentities = 0
    #def __init__(self):
        # self._out = cStringIO.StringIO()
        # HTMLParser.__init__(self)

    def parse(self, text, noentities=None):
        self._out = cStringIO.StringIO()
        self._parser_text = text
        self._parser_pos = self._writer_pos = pos = 0
        if noentities is None:
            noentities = self.noentities
        if noentities:
            rx = rxhtmlelementsnoentities
        else:
            rx = rxhtmlelements
        for m in rx.finditer(text):

            self._parser_start = m.start()
            self._parser_end = m.end()

            # print m.groupdict()
            result = None

            data = text[pos:m.start()]
            if data:
                result = self.handle_data(data)
                if result is None:
                    result = data
                self._out.write(result)

            data = m.group()
            if data.startswith("<"):
                if m.group("tag") is not None:
                    tagname = m.group("tag").lower()
                    if m.group("closetag") is not None:
                        func = getattr(self, "end_%s" % tagname, None)
                        if func is None:
                            result = self.handle_endtag(tagname)
                        else:
                            result = func()
                    else:
                        attr = self._getattrs(m.group("attrs"))
                        func = getattr(self, "start_%s" % tagname, None)
                        if func is None:
                            result = self.handle_starttag(tagname, attr)
                        else:
                            result = func(attr)
                elif m.group("comment") is not None:
                    result = self.handle_comment(m.group("comment"))
                elif m.group("script") is not None:
                    result = self.handle_script(self._getattrs(m.group("scriptattrs")), m.group("scriptdata"))
                elif m.group("style") is not None:
                    result = self.handle_style(self._getattrs(m.group("styleattrs")), m.group("styledata"))
                elif m.group("doctype") is not None:
                    result = self.handle_doctype(m.group("doctypeattrs"))
            else:
                ent = m.group("entity")
                if ent.startswith("#"):
                    try:
                        result = self.handle_charref(int(ent[1:]))
                    except:
                        pass
                else:
                    result = self.handle_entityref(ent)

            # WRITER
            if result is None:
                result = data

            self._out.write(result)
            self._writer_pos = m.end()
            self._parser_pos = pos = m.end()

        # WRITER
        self._out.write(text[self._writer_pos:pos])
        self._writer_pos = pos
        result = None

        data = text[pos:]
        if data:
            result = self.handle_data(data)
            if result is None:
                result = data
            self._out.write(result)

        #~ # WRITER
        #~ if result is not None:
            #~ self._out.write(result)
        #~ else:
            #~ self._out.write(text[self._writer_pos:])

        self._out.write(self.handle_finish())
        return self

    def handle_finish(self):
        return ""

    def write(self, value):
        self._out.write(str(value))

    def getvalue(self):
        try:
            return self._out.getvalue()
        except:
            return ""

    def __str__(self):
        return self.getvalue()

    def work(self, text):
        # self._out = cStringIO.StringIO()
        self.parse(text)
        return self._out.getvalue()

class IndentWriter(HTMLWriter):

    def __init__(self, space=" "):
        self.indent = 0
        self.ignore = 0
        self.space = space

    blocktags = "title head html form body h1 h2 h3 h4 h5 h6 p div table th tr td ul ol dl li dt dd".split()
    ignoretags = "textarea pre".split()
    rxwhite = re.compile("([\s\n\r\t\f]+)", re.MULTILINE|re.DOTALL)

    def handle_starttag(self, tag, attr):
        if tag in self.ignoretags:
            self.ignore += 1
        elif tag in self.blocktags:
            self.indent += 1
        return start_tag(tag, attr)

    def handle_endtag(self, tag):
        if tag in self.ignoretags:
            self.ignore -= 1
        elif tag in self.blocktags:
            self.indent -= 1
        return end_tag(tag)

    def _indent(self, m):
        v = m.group()
        if ("\n" in v) or ("\r" in v):
            return '\n' + (self.indent * self.space)
        return self.space

    def handle_data(self, value):
        if value and not self.ignore:
            return self.rxwhite.sub(self._indent, value)

    def handle_comment(self, data):
        return ""