# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.3 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/03/28 12:09:10 $"

import getopt
import sys
import os
import os.path
import pml

VERSION = "1.2.2"
VERSION_STR = "pisa %s\n(c) 2004-2007 spirito GmbH, Germany" % VERSION

USAGE = (VERSION_STR + """

USAGE: pisa [options] <src> <dest>

<src>
  Name of the PML file or "-" if you like to take
  the input from stdin.

<dest>
  Name of the generated PDF file or "-" if you like
  to send the result to stdout. Take care that the
  destination file is not alread opened by an other
  application like the AvrobatReader.

[options]
  --benchmark, -b:
    some speed informations
  --copyright:
    copyright and contact informations
  --debug, -d:
    paint frames in PDF
  --help, -h:
    shows this help text
  --parse, -p:
    Prints the parsed sourcecode
  --quiet, -q:
    No messages
  --start-viewer, -s:
    start PDF default viewer on Windows and MacOSX
    (e.g. AcrobatReader)
  --version:
    version information
  --warn, -w:
    show warnings
""").strip()

COPYRIGHT = VERSION_STR

BENCH = """
BENCHMARKING:
parsing   %.4f sec
document  %.4f sec
create    %.4f sec
""".strip()

def usage():
    print USAGE

def main():
    import os
    import os.path

    try:
        opts, args = getopt.getopt(sys.argv[1:], "bdhpqsw", [
            "benchmark",
            "quiet",
            "help",
            "start-viewer",
            "debug",
            "copyright",
            "version",
            "warn",
            "booklet=",
            "multivalent=",
            "multivalent-path=",
            "parse",
            "tempdir",
            ])
    except getopt.GetoptError:
        usage()
        sys.exit(2)

    startviewer = 0
    quiet = 0
    bench = 0
    debug = 0
    warn = 0
    tidy = 0
    multivalent_path = ""
    booklet = ""
    parse = 0
    tempdir = None

    for o, a in opts:
        if o in ("-h", "--help"):
            # Hilfe anzeigen
            usage()
            sys.exit()

        if o in ("-s", "--start-viewer"):
            # Anzeigeprogramm starten
            startviewer = 1

        if o in ("-q", "--quiet"):
            # Output unterdrcken
            quiet = 1

        if o in ("-b", "--benchmark"):
            # Zeiten ausgeben
            bench = 1

        if o in ("-d", "--debug"):
            # Debug
            debug = 1

        if o in ("-w", "--warn"):
            # Warnings
            warn = 1

        if o in ("--multivalent", "--multivalent-path"):
            # Multivalent.jar fr Booklet
            multivalent_path = a

        if o in ("--booklet",):
            # Booklet
            booklet = a

        if o in ("--copyright", "--version"):
            print COPYRIGHT
            sys.exit(0)

        if o in ("-p", "--parse"):
            # Print parse
            parse = 1

        if o in ("--tempdir",):
            # Tempdir
            tempdir = a

    if len(args) not in (1, 2):
        usage()
        sys.exit(2)

    if len(args)==2:
        src, dest = args
    else:
        src = args[0]
        dest = src + ".pdf"
        for i in range(10):
            try:
                open(dest, "wb").close()
                break
            except:
                pass
            dest = src + "-%d.pdf" % i

    if src=="-":
        fsrc = sys.stdin
        wpath = os.getcwd()
    else:
        fsrc = open(src, "r")
        wpath = os.path.abspath(src)

    # Just parse BEGIN

    if parse:
        # print "JUST PARSE"
        import htmlparser
        data = htmlparser.IndentWriter().work(fsrc.read().strip())
        print htmlparser.IndentWriter().work(data)
        sys.exit(0)

    # Just parse END

    fdestclose = 0
    if dest=="-":
        if sys.platform == "win32":
            import os, msvcrt
            msvcrt.setmode(sys.stdout.fileno(), os.O_BINARY)
        fdest = sys.stdout
        startviewer = 0
    else:
        dest = os.path.abspath(dest)
        try:
            open(dest, "wb").close()
        except:
            print "File '%s' seems to be in use of another application." % dest
            sys.exit(2)
        fdest = open(dest, "wb")
        fdestclose = 1

    #print wpath

    pdf = pml.CreatePDF(
        fsrc,
        fdest,
        debug = debug,
        path = wpath,
        errout = sys.stdout,
        log_warn = warn,
        log_err = 1,
        multivalent_path = multivalent_path,
        booklet = booklet,
        tempdir = tempdir,
        )

    if fdestclose:
        fdest.close()

    if pdf.log and not quiet:
        for mode, line, msg, code in pdf.log:
            print "%s in line %d: %s" % (mode, line, msg)

    if pdf.err:
        print "*** %d ERRORS OCCURED" % pdf.err
    else:
        if bench:
            print BENCH % tuple(pdf.bench)
        if startviewer:
            try:
                os.startfile(dest)
            except:
                # try to opan a la apple
                # cmd = 'nohup /Applications/Preview.app/Contents/MacOS/Preview "%s" &' % dest
                cmd = 'open "%s"' % dest
                # print cmd
                os.system(cmd)

if __name__=="__main__":
    main()
