# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by spirito GmbH, 2002       ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.8 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/04/05 13:39:06 $"

import htmlentitydefs

from pml_utils import *
from reportlab.lib.styles import *

ENTITIES = htmlentitydefs.entitydefs
NBSP = '\xa0'

PML_WARNING = "warning"
PML_ERROR = "error"
PML_EXCEPTION = "PML Exception"
PML_PREFIX = "pdf:"

CLASS   = 1
BOOL    = 2
FONT    = 3
COLOR   = 4
FILE    = 5
SIZE    = 6
INT     = 7
STRING  = 8
BOX     = 9
POS     = 10
STYLE   = 11

MUST    = 23

"""
Hier mssen alle Tags definiert werden und jedes Attribut.
Bei den Attributen wird entweder nur der Typ angegeben, oder
der Defaultwert, indem beide nacheinander in ein Tupel geschrieben
werden. Falls statt dem Defaultwert der Wert MUST eingetragen wird,
handelt es sich um einem Attribut das unbedingt angegeben werden mu.
"""

# definitionen der einzelnen tags

TAGS = {

    # FORMAT

    "document": (1, {
        "format":               (["a0", "a1", "a2", "a3", "a4", "a5", "a6",
                                  "b0", "b1", "b2", "b3", "b4", "b5", "b6",
                                  "letter", "legal", "elevenseventeen"], "a4"),
        "orientation":          ["portrait", "landscape"],
        "fullscreen":           (BOOL, "0"),
        "author":               (STRING, ""),
        "subject":              (STRING, ""),
        "title":                (STRING, ""),
        "duration":             INT,
        "showoutline":          (BOOL, "0"),
        "outline":              INT,
        }),

#    "class": (0, {
#        "parent":               (STRING, "default"),
#        "name":                 STRING,
#        "font":                 FONT,
#        "size":                 SIZE,
#        "leading":              SIZE,
#        "first":                SIZE,
#        "left":                 SIZE,
#        "right":                SIZE,
#        "before":               SIZE,
#        "after":                SIZE,
#        "color":                COLOR,
#        "bgcolor":              COLOR,
#        "align":                ["left", "right", "center", "justify"],
#        "bulletfont":           FONT,
#        "bulletsize":           SIZE,
#        "bulletindent":         SIZE,
#        "underlinecolor":       COLOR,
#        "bold":                 BOOL,
#        "underline":            BOOL,
#        "italic":               BOOL,
#        }),

    "link": (0, {
        "href":                (FILE, MUST),
        "rel":                 (STRING, "stylesheet"),
        "type":                (STRING, "text/css"),
        }),

#    "include": (0, {
#        "href":                 (FILE, MUST),
#        "first":                (INT, -1),
#        "last":                 (INT, -1),
#        }),

    "template": (1, {
        "name":                 (STRING, "default"),
        "format":               (["a0", "a1", "a2", "a3", "a4", "a5", "a6",
                                  "b0", "b1", "b2", "b3", "b4", "b5", "b6",
                                  "letter", "legal", "elevenseventeen"], "a4"),
        "orientation":          ["portrait", "landscape"],
        "background":           FILE,
        }),

    "frame": (0, {
        "name":                 STRING,
        "box":                  (BOX, MUST),
        "border":               (BOOL, "0"),
        }),

    "static": (1, {
        "name":                 STRING,
        "box":                  (BOX, MUST),
        "border":               (BOOL, "0"),
        }),

    "nexttemplate": (0, {
        "name":                 (STRING, "default"),
        }),

    "nextpage": (0, {
        "template":             STRING,
        # "background":           FILE,
        }),

    "nextframe": (0, {}),

    "fontembed": (0, {
        "afm":                  FILE,
        "pfb":                  FILE,
        "ttf":                  FILE,
        "name":                 STRING,
        "print":                (BOOL, "0"),
        "encoding":             (STRING, "WinAnsiEncoding"),
        "alias":                STRING,
        }),

    "drawline": (0, {
        "from":                 (POS, MUST),
        "to":                   (POS, MUST),
        "color":                (COLOR, "#000000"),
        "width":                (SIZE, 1),
        }),

    "drawpoint": (0, {
        "pos":                  (POS, MUST),
        "color":                (COLOR, "#000000"),
        "width":                (SIZE, 1),
        }),

    "drawlines": (0, {
        "coords":               (STRING, MUST),
        "color":                (COLOR, "#000000"),
        "width":                (SIZE, 1),
        }),

    "drawstring": (0, {
        "pos":                  (POS, MUST),
        "text":                 (STRING, MUST),
        "color":                (COLOR, "#000000"),
        "align":                (["left", "center", "right"], "right"),
        "valign":               (["top", "middle", "bottom"], "bottom"),
        "class":                CLASS,
        "rotate":               (INT, "0"),
        }),

    "drawimg": (0, {
        "pos":                  (POS, MUST),
        "src":                  (FILE, MUST),
        "width":                SIZE,
        "height":               SIZE,
        "align":                (["left", "center", "right"], "right"),
        "valign":               (["top", "middle", "bottom"], "bottom"),
        }),

    "img": (0, {
        "src":                  (FILE, MUST),
        "width":                SIZE,
        "height":               SIZE,
        "align":                (["left", "center", "right"], "left"),
        "class":                (CLASS, "img"),
        "style":                STYLE,
        }),

    "table": (1, {
        "align":                (["left", "center", "right"], "left"),
        "valign":               (["top", "bottom", "middle"], "middle"),
        #"left":                 SIZE,
        #"right":                SIZE,
        #"top":                  SIZE,
        #"bottom":               SIZE,
        "border":               (SIZE, "0"),
        "bordercolor":          (COLOR, "#000000"),
        "bgcolor":              COLOR,
        "cellpadding":          (SIZE, "1"),
        "repeat":               (INT, "0"),
        "width":                STRING,
        "class":                (CLASS, "table"),
        "style":                STYLE,
        }),

    "tr": (1, {
        "bgcolor":              COLOR,
        "valign":               ["top", "bottom", "middle"],
        "border":               SIZE,
        "bordercolor":          (COLOR, "#000000"),
        "class":                (CLASS, "tr"),
        "style":                STYLE,
        }),

    "td": (1, {
        "align":                ["left", "center", "right", "justify"],
        "valign":               ["top", "bottom", "middle"],
        "width":                STRING,
        "bgcolor":              COLOR,
        "border":               SIZE,
        "bordercolor":          (COLOR, "#000000"),
        "class":                (CLASS, "td"),
        "style":                STYLE,
        "colspan":		        INT, 
        "rowspan":		        INT,
        }),

    "th": (1, {
        "align":                ["left", "center", "right", "justify"],
        "valign":               ["top", "bottom", "middle"],
        "width":                STRING,
        "bgcolor":              COLOR,
        "border":               SIZE,
        "bordercolor":          (COLOR, "#000000"),
        "class":                (CLASS, "th"),
        "style":                STYLE,
        "colspan":		        INT, 
        "rowspan":		        INT,
        }),

    "dl": (1, {
        "class":                (CLASS, "dl"),
        "style":                STYLE,
        }),

    "dd": (1, {
        "class":                (CLASS, "dd"),
        "style":                STYLE,
        }),

    "dt": (1, {
        "class":                (CLASS, "dt"),
        "style":                STYLE,
        }),

    "ol": (1, {
        "type":                 (["1", "a", "A", "i", "I"], "1"),
        "class":                (CLASS, "ol"),
        "style":                STYLE,
        }),

    "ul": (1, {
        "class":                (CLASS, "ul"),
        "type":                 (["circle", "disk", "square"], "disk"),
        "style":                STYLE,
        }),

    "li": (1, {
        "class":                (CLASS, "li"),
        "style":                STYLE,
        }),

    "hr": (0, {
        "color":                (COLOR, "#000000"),
        "size":                 (SIZE, "1"),
        "class":                (CLASS, "hr"),
        "style":                STYLE,
        }),

    "pre": (1, {
        "class":                (CLASS, "pre"),
        "style":                STYLE,
        }),

    "div": (1, {
        "align":                ["left", "center", "right", "justify"],
        "class":                (CLASS, "div"),
        "style":                STYLE,
        }),

    "p": (1, {
        "align":                ["left", "center", "right", "justify"],
        "class":                (CLASS, "p"),
        "style":                STYLE,
        }),

    "br": (0, {
        "class":                (CLASS, "br"),
        "style":                STYLE,
        }),

    "h1": (1, {
        "class":                (CLASS, "h1"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    "h2": (1, {
        "class":                (CLASS, "h2"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    "h3": (1, {
        "class":                (CLASS, "h3"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    "h4": (1, {
        "class":                (CLASS, "h4"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    "h5": (1, {
        "class":                (CLASS, "h5"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    "h6": (1, {
        "class":                (CLASS, "h6"),
        "style":                STYLE,
        "outline":              STRING,
        "closed":               (INT, 0),
        }),

    #"h7": (1, {
    #    "class":                (CLASS, "h7"),
    #    "style":                STYLE,
    #    "outline":              STRING,
    #    "closed":               (INT, 0),
    #    }),

    "strong": (1, {
        "class":                (CLASS, "strong"),
        "style":                STYLE,
        }),

    "b": (1, {
        "class":                (CLASS, "b"),
        "style":                STYLE,
        }),

    "em": (1, {
        "class":                (CLASS, "em"),
        "style":                STYLE,
        }),

    "i": (1, {
        "class":                (CLASS, "i"),
        "style":                STYLE,
        }),

    "u": (1, {
        "class":                (CLASS, "u"),
        "style":                STYLE,
        }),

    "span": (1, {
        "class":                (CLASS, "span"),
        "style":                STYLE,
        }),
        
    "font": (1, {
        "face":                 FONT,
        "color":                COLOR,
        "size":                 SIZE,
        }),

    "sup": (1, {
        "class":                (CLASS, "sup"),
        "style":                STYLE,
        }),

    "super": (1, {
        "class":                (CLASS, "super"),
        "style":                STYLE,
        }),

    "sub": (1, {
        "class":                (CLASS, "sub"),
        "style":                STYLE,
        }),

    "spacer" : (0, {
        "height":               (SIZE, MUST),
        }),

    "pagenumber": (0, {
        "example":              (STRING, "0"),
        }),

    "a": (1, {
        "href":                 STRING,
        "name":                 STRING,
        "class":                (CLASS, "a"),
        "style":                STYLE,
        }),

    "span": (1, {
        "class":                (CLASS, "span"),
        "style":                STYLE,
        }),
        
    "version": (0, {     
        }),
    
    # The chart example, see pml_charts
    #"chart": (0, {
    #    "strokecolor":          (COLOR, "#000000"),
    #    }),

    "keeptogether": (1, {      
        }),

    "keepinframe": (1, {    
        "maxwidth":            SIZE,
        "maxheight":           SIZE,
        "mergespace":          (INT, 1),
        "mode":                (["error", "continue", "shrink", "truncate"], "shrink"),
        "name":                (STRING, "")
        }),
         
    }

# pdf compat

PML_WITHPREFIX = [
    "document",
    "template",
    "frame",
    "static",
    "nexttemplate",
    "nextframe",
    "nextpage",
    "fontembed",
    "spacer",
    "pagenumber",
    "class",
    "drawline",
    "drawlines",
    "drawimg",
    "drawpoint",
    "drawstring"
    ]

if 0:
    import copy
    for tag in TAGS.keys():
        try:
            if tag.startswith(PML_PREFIX):
                print "tag", tag
                TAGS[tag[len(PML_PREFIX):]] = copy.copy(TAGS[tag])
        except Exception, e:
            print e
    print TAGS["div"]

# eingebaute schriftarten

FONTS = {
    "courier": "Courier",
    "courier-bold": "Courier-Bold",
    "courier-boldoblique": "Courier-BoldOblique",
    "courier-oblique": "Courier-Oblique",
    "helvetica": "Helvetica",
    "helvetica-bold": "Helvetica-Bold",
    "helvetica-boldoblique": "Helvetica-BoldOblique",
    "helvetica-oblique": "Helvetica-Oblique",
    "times": "Times",
    "times-bold": "Times-Bold",
    "times-boldoblique": "Times-BoldOblique",
    "times-oblique": "Times-Oblique",
    "symbol": "Symbol",
    "zapfdingbats": "ZapfDingbats",
    "zapf-dingbats": "ZapfDingbats",
    
    # Alias
    "arial": "Helvetica",
    "times new roman": "Times",
    "georgia": "Times",
    'serif':'Times',
    'sansserif':'Helvetica',
    'monospaced':'Courier', 
    'courier new':'Courier',    
    'verdana':'Helvetica',
    'geneva':'Helvetica',
    }

from reportlab.lib.colors import *

def MakeDefaultStyles():
    """
    Some default styles to be overridden by style definitions
    """
    defstyle = PmlStyle("default", None, {
        # 'fontName': "Helvetica",
        'fontSize': 10.0,
        'leading': 12.0,  
        'textColor': black,      
        })
    return {
        "default" :
            defstyle,
        #"body" : 
        #   PmlStyle("body", defstyle),
        "p" :
            PmlStyle("p", defstyle),
        "div" :
            PmlStyle("div", defstyle),
        "h1" : PmlStyle("h1", defstyle, {
            'bold': 1,
            'fontSize': 18,
            'leading': 22,
            'spaceBefore': 20,
            'spaceAfter': 6,
            }),
        "h2" : PmlStyle("h2", defstyle, {
            'bold': 1,
            'fontSize': 14,
            'leading': 18,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "h3" : PmlStyle("h3", defstyle, {
            'bold': 1,
            'italic': 1,
            'fontSize': 12,
            'leading': 14,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "h4" : PmlStyle("h4", defstyle, {
            'bold': 1,
            'italic': 1,
            'fontSize': 12,
            'leading': 14,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "h5" : PmlStyle("h5", defstyle, {
            'bold': 1,
            'italic': 1,
            'fontSize': 12,
            'leading': 14,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "h6" : PmlStyle("h6", defstyle, {
            'bold': 1,
            'italic': 1,
            'fontSize': 12,
            'leading': 14,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "h7" : PmlStyle("h7", defstyle, {
            'bold': 1,
            'italic': 1,
            'fontSize': 12,
            'leading': 14,
            'spaceBefore': 12,
            'spaceAfter': 6,
            }),
        "pre" : PmlStyle("pre", defstyle, {
            'fontName':'Courier',
            'spaceBefore': 6,
            'spaceAfter': 6,
            }),
        "ul" : PmlStyle("ul", defstyle, {
            'bulletIndent': 8,
            }),
        "ol" : PmlStyle("ul", defstyle, {
            'bulletIndent': 16,
            }),
        "a" : PmlStyle("a", None, {
            'underline': 1,
            'textColor': blue,
            }),
        "b" : PmlStyle("b", None, {
            'bold': 1,
            }),
        "i" : PmlStyle("i", None, {
            'italic': 1,
            }),
        "strong" : PmlStyle("strong", None, {
            'bold': 1,
            }),
        "em" : PmlStyle("em", None, {
            'italic': 1,
            }),
        "u" : PmlStyle("u", None, {
            'underline': 1,
            }),
        "super" : PmlStyle("super", None, {
            'super': 1,
            }),
        "sub" : PmlStyle("sub", None, {
            'sub': 1,
            }),
        "dl" : PmlStyle("dl", None, {
            }),
        "img" : PmlStyle("img", None),
        "br" : PmlStyle("br", None),
        "hr" : PmlStyle("hr", None),
        "th" : PmlStyle("th", defstyle),
        "td" : PmlStyle("td", defstyle),
        "table" : PmlStyle("table", defstyle),
        "tr" : PmlStyle("tr", defstyle),
        "span" : PmlStyle("span", defstyle),
        }

