# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.2 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/01/12 13:07:05 $"

import copy
import tempfile
import string
import os

from reportlab.lib.units import inch, cm
from reportlab.lib.styles import *
from reportlab.lib.enums import *
from reportlab.lib.colors import *
from reportlab.lib.pagesizes import *
from reportlab.lib.utils import *
from reportlab.pdfbase import pdfmetrics
from reportlab.platypus import *
from reportlab.platypus.flowables import Flowable
import reportlab.graphics.shapes as shapes

from pml_defs import *
from pml_utils import *

class PDFDraw:

    def start_drawline(self, a):
        o = shapes.Line(
            a["from"][0], 
            a["from"][1], 
            a["to"][0], 
            a["to"][1], 
            strokeColor=a["color"], 
            strokeWidth=a["width"], 
            )
        self._drawing.add(o)

    def start_drawpoint(self, a):
        o = shapes.Circle(
            a["pos"][0], 
            a["pos"][1], 
            1, 
            strokeColor=a["color"], 
            strokeWidth=a["width"], 
            )
        self._drawing.add(o)

    def start_drawlines(self, a):
        data = a["coords"].split()
        if len(data)<4:
            self.error("drawlines coords must contain at least 4 values")
        if len(data)%2!=0:
            self.error("drawlines coords must contain an even number of values")
        for i in range(0, len(data), 2):
            data[i:i+2] = getPos(data[i] + " " + data[i+1], self._pagesize)
        o = shapes.PolyLine(
            data, 
            strokeColor=a["color"], 
            strokeWidth=a["width"], 
            )
        self._drawing.add(o)

    def start_drawstring(self, a):

        style = PmlStyleObj("drawstring", preset=self.styles["default"].collect())
        if a["class"] is not None:
            style = PmlStyleObj("drawstring", preset=self.styles[a["class"]].collect())
        #~ style.dump()

        h = style["fontSize"] * 0.8
        x, y = a.pos

        anchor = "start"
        if a.align=="left":
            anchor = "end"
        elif a.align=="center":
            anchor = "middle"
        if a.valign=="bottom":
            y -= h
        elif a.align=="middle":
            y -= h/2

        o = shapes.Group(shapes.String(
            0, 
            0, 
            toString(a["text"]), 
            textAnchor = anchor, 
            fontName = style["fontName"], 
            fontSize = style["fontSize"], 
            fillColor = style["textColor"], 
            ))
        o.translate(x, y)
        o.rotate(a.rotate)
        self._drawing.add(o)

    def start_drawimg(self, a):
        if os.path.isfile(a.src):

            iw, ih = ImageReader(a.src).getSize()

            w = a.width
            h = a.height
            x, y = a.pos

            if (w is None) and (h is not None):
                factor = float(h) / ih
                w = iw * factor
            elif (h is None) and (w is not None):
                factor = float(w) / iw
                h = ih * factor
            elif h is None and w is None:
                w = iw
                h = ih

            if a.align=="left":
                x -= w
            elif a.align=="center":
                x -= w/2
            if a.valign=="bottom":
                y -= h
            elif a.align=="middle":
                y -= h/2

            o = shapes.Image(
                x, 
                y, 
                w, 
                h, 
                a.src, 
                )

            self._drawing.add(o)

            #~ except Exception, e:
                #~ self.warning("drawimg: " + str(e))

        else:
            self.warning("File '%s' does not exist." % a.src)
