# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.7 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/03/15 17:33:21 $"

"""
Alles was so mit dem Dokument und seinem Aufbau zu tun hat:

 - document
 - frame
 - static
 - template
 - nextframe
 - nexttemplate
 - nextpage
 - background
 - drawline
"""

import copy
import tempfile
import string
import os

from reportlab.lib.units import inch, cm
from reportlab.lib.styles import *
from reportlab.lib.enums import *
from reportlab.lib.colors import *
from reportlab.lib.pagesizes import *
from reportlab.pdfbase import pdfmetrics
from reportlab.platypus import *
from reportlab.platypus.flowables import Flowable
#~ from reportlab.graphics.shapes import *
import reportlab.graphics.shapes as shapes

from pml_defs import *
from pml_utils import *

class PmlPageDrawing(shapes.Drawing):

    def __init__(self, pagesize):
        shapes.Drawing.__init__(self, pagesize[0], pagesize[1])

    def draw(self, canvas):
        self.canv = canvas
        shapes.Drawing.draw(self)

class PmlPageTemplate(PageTemplate):

    def beforeDrawPage(self, canvas, doc):
        canvas.saveState()
        try:
            # Paint static frames
            for frame, story in self.pml_statics:
                story = copy.deepcopy(story)
                frame = copy.deepcopy(frame)
                frame.addFromList(story, canvas)
        except Exception, e:
            #~ print str(e)
            pass
        # canvas.restoreState()

        try:
            self.pml_bgstory.append(self.pml_background)
        except:
            self.pml_bgstory.append(None)            

        # canvas.saveState()
        try:
            self.pml_drawing.draw(canvas)
        except Exception, e:
            # print "drawing exception", str(e)
            pass
        canvas.restoreState()

class PmlPageNumberObject(PageNumberObject):
    pass

    #def __init__(self, example="XXX"):
    #    self.example = example # XXX SHOULD ADD THE ABILITY TO PASS IN EXAMPLES

    #def width(self, engine):
    #    from reportlab.pdfbase.pdfmetrics import stringWidth
    #    return stringWidth(self.example, engine.fontName, engine.fontSize)

    #def execute(self, engine, textobject, canvas):
    #    n = canvas.getPageNumber()
    #    textobject.textOut(str(n))

class MyEffect(Flowable):

    def __init__(self, effect, duration, direction, dimension, motion):
        self._effect = effect
        self._duration = duration
        self._direction = direction
        self._dimension = dimension
        self._motion = motion

    def wrap(self, availWidth, availHeight):
        return (0, 0)

    def draw(self):
        self.canv.setPageTransition(
            self._effect, 
            self._duration, 
            self._direction, 
            self._dimension, 
            self._motion)

PML_PAGESIZES = {
        "a0": A0, 
        "a1": A1, 
        "a2": A2, 
        "a3": A3, 
        "a4": A4, 
        "a5": A5, 
        "a6": A6, 
        "b0": B0, 
        "b1": B1, 
        "b2": B2, 
        "b3": B3, 
        "b4": B4, 
        "b5": B5, 
        "b6": B6, 
        "letter": LETTER, 
        "legal": LEGAL, 
        "elevenseventeen": ELEVENSEVENTEEN, 
        "1117": ELEVENSEVENTEEN, 
        }

class PDFFormat:

    def end_document(self): pass
    def start_document(self, attrs):
        # OKAY

        self._ddata["author"] = attrs.author
        self._ddata["subject"] = attrs.subject
        self._ddata["title"] = attrs.title
        self._ddata["fullscreen"] = attrs.fullscreen
        self._ddata["duration"] = attrs.duration
        self._ddata["showoutline"] = attrs.showoutline

        self._pagesize = PML_PAGESIZES[attrs.format]
        if attrs.orientation is not None:
            if attrs.orientation == "landscape":
                self._pagesize = landscape(self._pagesize)
            elif attrs.orientation == "portrait":
                self._pagesize = portrait(self._pagesize)

        if attrs.outline is not None:
            #self._outline = 3
            self._outline = attrs.outline
            # print repr(attrs.outline)
            #~ self.add("outline" + str(self._outline))

    def start_frame(self, attrs):
        #x, y, w, h = self.getCoords(attrs, self._pagesize)
        x, y, w, h = attrs.box
        name = attrs.name
        if name is None:
            name = str(self.UID())
        self._frames.append(Frame(x, y, w, h, id=name, leftPadding=0, rightPadding=0, bottomPadding=0, topPadding=0, showBoundary=attrs.border))

    def start_static(self, attrs):
        #x, y, w, h = self.getCoords(attrs, self._pagesize)
        x, y, w, h = attrs.box
        name = attrs.name
        if name is None:
            name = str(self.UID())
        self.next_para()
        self._statics.append([Frame(x, y, w, h, id=name, leftPadding=0, rightPadding=0, bottomPadding=0, topPadding=0, showBoundary=attrs.border), []])
        self.story.append([])

    def end_static(self):
        self.add_noop()
        self.next_para()
        self._statics[-1][1] = self.story.pop()

    def start_template(self, attrs):
        # OKAY

        self.next_para()
        if self._templates.has_key(attrs.name):
            self.warning("template '%s' has already been defined" % attrs.name)

        self._templname = attrs.name
        self._templbackground = attrs.background
        self._oldpagesize = self._pagesize

        self._pagesize = PML_PAGESIZES[attrs.format]
        if attrs.orientation is not None:
            if attrs.orientation == "landscape":
                self._pagesize = landscape(self._pagesize)
            elif attrs.orientation == "portrait":
                self._pagesize = portrait(self._pagesize)

        self._drawing = PmlPageDrawing(self._pagesize)

    def end_template(self):
        if len(self._frames)==0:
            raise PML_EXCEPTION, "missing frame definitions for template"
        pt = PmlPageTemplate(id=self._templname, frames=self._frames, pagesize=self._pagesize)
        pt.pml_statics = self._statics
        pt.pml_draw = self._draw
        pt.pml_drawing = self._drawing
        pt.pml_background = self._templbackground
        pt.pml_bgstory = self._bgstory
        self._drawing = None
        self._templates[self._templname] = pt
        self._frames = []
        self._statics = []
        self._draw = []
        self._templname = ""
        self._pagesize = self._oldpagesize
        self.next_para()
        
    def start_nextframe(self, attrs):
        self.next_para()
        self.add_story(FrameBreak())

    #def start_include(self, attrs):
    #    self.next_para()
    #    self.add_story(FrameBreak())

    def start_nexttemplate(self, attrs):
        self.add_story(NextPageTemplate(attrs.name))

    def start_nextpage(self, attrs):
        self.next_para()
        if attrs.template is not None:
            self.add_story(NextPageTemplate(attrs.template))
        self.add_story(PageBreak())

    def start_effect(self, attrs):
        # OKAY
        self.add_story(
            MyEffect(
                attrs.name.capitalize(), 
                attrs.duration, 
                int(attrs.direction), 
                attrs.dimension.upper(), 
                attrs.motion.upper()
                ))

    def start_pagenumber(self, attrs):
        self.program.append(PmlPageNumberObject(example=attrs.example))

    def end_pagenumber(self): 
        pass

    def start_version(self, attrs):
        import main
        self.add(main.VERSION_STR)
        