# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.2 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/01/11 12:18:57 $"

from pml_utils import *

class PmlHorizontalRule(Flowable):

    def __init__(self, color, size):
        self._color = color
        self._size = size

    def wrap(self, availWidth, availHeight):
        self.width = availWidth
        return (availWidth, self._size + 2)

    def draw(self):
        canvas = self.canv
        canvas.setLineWidth(self._size)
        canvas.setFillColor(self._color)
        canvas.setStrokeColor(self._color)
        canvas.line(0, 1, self.width, 1)

class PDFSimple:

    def start_pre(self, attrs):
        self.next_para(attrs, "pre")
        self._pre += 1

    def end_pre(self):
        #t = string.rstrip(string.join(self.out,""))
        #self.out = []
        #if t:
        #    s = self.styles["default"].copy()
        #    if self.nextstyle:
        #        s.update(self.styles[self.nextstyle])
#            pstyle = ParagraphStyle(self.nextstyle, **s)
        #    pstyle = apply(ParagraphStyle, [self.nextstyle], s)
        #    self.add_story(XPreformatted(t, pstyle))
        self.next_para(None)
        self._pre -= 1

    def start_hr(self, attrs):
        self.next_para()
        self.add_story(PmlHorizontalRule(
            attrs.color,
            attrs.size
            ))
        self.next_para()

    def start_p(self, attrs):
        self.next_para(attrs, "p")

    def end_p(self):
        pass

    def start_div(self, attrs):
        self.next_para(attrs, "div")

    def end_div(self):
        pass

    def start_br(self, attrs):
        self.add_noop()
        self.program.append(("nextLine", 0))

    def start_font(self, attrs):
        h = {}
        if attrs.face is not None:
            h["fontName"] = attrs.face
        if attrs.size is not None:
            h["fontSize"] = attrs.size
            h["leading"] = attrs.size * 1.2
        if attrs.color is not None:
            h["textColor"] = attrs.color
        self.push_style(PmlStyle(preset=h))

    def end_font(self):
        self.pop_style()

    def start_span(self, attrs):
        self.push_style(self.styles["span"])

    def end_span(self):
        self.pop_style()
        
    def start_u(self, attrs):
        self.push_style(self.styles["u"])

    def end_u(self):
        self.pop_style()

    def start_b(self, attrs):
        self.push_style(self.styles["b"])

    def end_b(self):
        self.pop_style()

    def start_strong(self, attrs):
        self.push_style(self.styles["strong"])

    def end_strong(self):
        self.pop_style()

    def start_i(self, attrs):
        self.push_style(self.styles["i"])

    def end_i(self):
        self.pop_style()

    def start_em(self, attrs):
        self.push_style(self.styles["em"])

    def end_em(self):
        self.pop_style()

    #def start_greek(self, attrs): 
    #    self.trough("greek", attrs)
        
    #def end_greek(self): 
    #    self.trough("greek")

    def start_super(self, attrs):
        self.push_style(self.styles["super"])

    def end_super(self):
        self.pop_style()

    start_sup, end_sup = start_super, end_super

    def start_sub(self, attrs):
        self.push_style(self.styles["sub"])

    def end_sub(self):
        self.pop_style()

    def start_spacer(self, attrs):
        self.next_para()
        self.add_story(Spacer(width=1, height=attrs.height))

    def start_span(self, attrs):
        st = self.styles.get(attrs["class"], self.styles["default"])
        self.push_style(st)

    def end_span(self):
        self.pop_style()

