# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.10 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/01/12 14:41:21 $"

from pml_utils import *

import re
import sys
import cStringIO

import reportlab.rl_config
reportlab.rl_config.warnOnMissingFontGlyphs = 0

from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont
from reportlab.lib.fonts import addMapping

from pml_utils import *

PAT_CSS = re.compile("""
    ([a-zA-Z\-\_\.\:\#0-9]+)
        \s*?
        \{
            ([^\}]*)
        \}
    """, re.DOTALL|re.MULTILINE|re.IGNORECASE|re.VERBOSE)
PAT_CSS_COMMENT = re.compile("\s*<!-[-]+(?P<css>.*?)[-]+->\s*", re.DOTALL|re.MULTILINE|re.IGNORECASE)
PAT_CSS_REM1 = re.compile("""\/\*.*?\*\/""", re.DOTALL|re.MULTILINE|re.IGNORECASE|re.VERBOSE)
PAT_CSS_REM2 = re.compile("""\/\/.*?$""", re.DOTALL|re.MULTILINE|re.IGNORECASE|re.VERBOSE)

class PDFStyles:

    def ParseStyles(self, value, name=None):
        global PAT_CSS
        global PAT_CSS_COMMENT
        global PAT_CSS_REM1
        global PAT_CSS_REM2

        if not name:
            name = "__tmp__" + str(self.UID())

        h = PmlStyle(name, self.styles["default"])
        top = bottom = left = right = None
        bbleft = bbright = bbbottom = bbtop = None
        bcleft = bcright = bcbottom = bctop = None
        bsleft = bsright = bsbottom = bstop = None

        for part in value.split(";"):
            part = part.strip()
            if part:
                p, v = part.split(":")
                nn = p.strip()
                dd = v.strip()

                try:

                    nn = nn.lower().strip()
                    #print "STYLE:", nn, dd

                    if nn=="display":
                        h["display"] = (dd.lower() != "none")

                    elif nn=="width":
                        h["width"] = getSize(dd)

                    elif nn=="height":
                        h["height"] = getSize(dd)

                    elif nn=="font-size":
                        h["fontSize"] = getSize(dd)

                    elif nn=="font-family":
                        h["fontName"] = getFontname(self._fonts, dd)

                    elif nn=="color":
                        h["textColor"] = getColor(dd)

                    elif nn=="line-height":
                        h['leading'] = getSize(dd)

                    elif nn=="background-color":
                        h["backColor"] = getColor(dd)

                    elif nn=="text-decoration":
                        vv = dd.lower().split()
                        if "underline" in vv:
                            h['underline'] = 1
                        if "none" in vv:
                            h['underline'] = 0

                    elif nn=="font-weight":
                        if dd in ("bold", "bolder", "100", "200", "300",
                            "400", "500", "600", "700", "800", "900"):
                            h['bold'] = 1
                        else:
                            h['bold'] = 0

                    elif nn=="text-align":
                        h['alignment'] = ["left", "center", "right",
                            "", "justify"].index(dd)
                            
                    elif nn=="vertical-align":
                        h['verticalAlign'] = dd #["left", "center", "right","", "justify"].index(dd)

                    elif nn=="text-indent":
                        h['firstLineIndent'] = getSize(dd)

                    elif nn=="font-style":
                        if dd in ("italic", "oblique"):
                            h['italic'] = 1
                        else:
                            h['italic'] = 0

                    # Margins
                    elif nn=="margin":
                        vv = dd.lower().split()
                        vv = map(getSize, vv)
                        if len(vv)==1:
                            top = bottom = left = right = vv[0]
                        elif len(vv)==2:
                            top = bottom = vv[0]
                            left = right = vv[1]
                        elif len(vv)==3:
                            top = vv[0]
                            left = right = vv[1]
                            bottom = vv[2]
                        elif len(vv)>3:
                            top = vv[0]
                            right = vv[1]
                            bottom = vv[2]
                            left = vv[3]
                    elif nn=="margin-left":
                        left = getSize(dd)
                    elif nn=="margin-right":
                        right = getSize(dd)
                    elif nn=="margin-top":
                        top = getSize(dd)
                    elif nn=="margin-bottom":
                        bottom = getSize(dd)

                    # Paddings
                    elif nn=="padding":
                        vv = dd.lower().split()
                        vv = map(getSize, vv)
                        if len(vv)==1:
                            top = bottom = left = right = vv[0]
                        elif len(vv)==2:
                            top = bottom = vv[0]
                            left = right = vv[1]
                        elif len(vv)==3:
                            top = vv[0]
                            left = right = vv[1]
                            bottom = vv[2]
                        elif len(vv)>3:
                            top = vv[0]
                            right = vv[1]
                            bottom = vv[2]
                            left = vv[3]
                    elif nn=="padding-left":
                        left = getSize(dd)
                    elif nn=="padding-right":
                        right = getSize(dd)
                    elif nn=="padding-top":
                        top = getSize(dd)
                    elif nn=="padding-bottom":
                        bottom = getSize(dd)
                        
                    # Border width
                    elif nn=="border-width":
                        bbleft = bbright = bbbottom = bbtop = getSize(dd)
                    elif nn=="border-top-width":
                        bbtop = getSize(dd)
                    elif nn=="border-bottom-width":
                        bbbottom = getSize(dd)
                    elif nn=="border-left-width":
                        bbleft = getSize(dd)
                    elif nn=="border-right-width":
                        bbright = getSize(dd)

                    # Border color
                    elif nn=="border-color":
                        bcleft = bcright = bcbottom = bctop = getColor(dd)
                    elif nn=="border-top-color":
                        bctop = getColor(dd)
                    elif nn=="border-bottom-color":
                        bcbottom = getColor(dd)
                    elif nn=="border-left-color":
                        bcleft = getColor(dd)
                    elif nn=="border-right-color":
                        bcright = getColor(dd)

                    # Border style
                    elif nn=="border-style":
                        bsleft = bsright = bsbottom = bstop = dd
                    elif nn=="border-top-style":
                        bstop = dd
                    elif nn=="border-bottom-style":
                        bsbottom = dd
                    elif nn=="border-left-style":
                        bsleft = dd
                    elif nn=="border-right-style":
                        bsright = dd
                        
                    # Border complete
                    elif nn=="border":
                        vv = dd.split()
                        if len(vv)>=3:
                            bbleft = bbright = bbbottom = bbtop = getSize(vv[0])
                            bcleft = bcright = bcbottom = bctop = getColor(vv[2])
                            bsleft = bsright = bsbottom = bstop = vv[1]
                    elif nn=="border-top":
                        vv = dd.split()
                        if len(vv)>=3:
                            bbtop = getSize(vv[0])
                            bctop = getColor(vv[2])
                            bstop = vv[1]
                    elif nn=="border-bottom":
                        vv = dd.split()
                        if len(vv)>=3:
                            bbbottom = getSize(vv[0])
                            bcbottom = getColor(vv[2])
                            bsbottom = vv[1]
                    elif nn=="border-left":
                        vv = dd.split()
                        if len(vv)>=3:
                            bbleft = getSize(vv[0])
                            bcleft = getColor(vv[2])
                            bsleft = vv[1]
                    elif nn=="border-right":
                        vv = dd.split()
                        if len(vv)>=3:
                            bbright = getSize(vv[0])
                            bcright = getColor(vv[2])
                            bsright = vv[1]

                except Exception, e:
                    self.warning("Unknown style %s: %s (%s)" % (str(nn), str(dd), str(e)))

                # print "   ", nn, dd

            # h.dump()

        # Paddings and margins (combined)
        if top is not None:
            h["spaceBefore"] = top
            h["paddingTop"] = top
        if bottom is not None:
            h["spaceAfter"] = bottom
            h["paddingBottom"] = bottom
        if left is not None:
            h["leftIndent"] = left
            h["paddingLeft"] = left
        if right is not None:
            h["rigthIndent"] = -right
            h["paddingRight"] = -right

        # If fonsize exceeds leading adjust it automatically
        if h['leading']<h['fontSize']:
            self.warning("leading is smaller than fontsize")
            h['leading'] = h['fontSize'] * 1.2

        #print bbleft, bbright, bbbottom, bbtop, bcleft, bcright, bcbottom, bctop

        # Border definitions
        h["borderColorLeft"] = bcleft
        h["borderColorRight"] = bcright
        h["borderColorTop"] = bctop
        h["borderColorBottom"] = bcbottom

        h["borderWidthLeft"] = bbleft
        h["borderWidthRight"] = bbright
        h["borderWidthTop"] = bbtop
        h["borderWidthBottom"] = bbbottom

        h["borderStyleLeft"] = bsleft
        h["borderStyleRight"] = bsright
        h["borderStyleTop"] = bstop
        h["borderStyleBottom"] = bsbottom
        
        #if self.debug:
        #    h.dump()

        return h

    def handle_style(self, attr, data):
        global PAT_CSS
        global PAT_CSS_COMMENT
        global PAT_CSS_REM1
        global PAT_CSS_REM2

        css = data
        m = PAT_CSS_COMMENT.match(data)
        if m:
            css = m.group("css")

        css = PAT_CSS_REM1.sub("", css)
        css = PAT_CSS_REM2.sub("", css)

        for name, value in PAT_CSS.findall(css):
            name = name.lower().strip()
            if name[0]==".":
                name = name[1:]
            self.styles[name] = self.ParseStyles(value, name)

    def start_link(self, attrs):
        # print attrs["href"]
        data = open(attrs["href"], "r").read()
        self.handle_style(attrs, data)

    def _registerFont(self, fontname, alias=""):
        self._fonts[fontname.lower()] = fontname
        for a in alias.split(","):
            a = a.strip().lower()
            if a:
                self._fonts[a] = fontname

    def start_fontembed(self, attrs):

        fontAlias = attrs.get("alias", "")

        try:
            #~ print attrs.name, attrs.ttf, attrs.afm, attrs.pfb

            if attrs.ttf is not None:
                ttf = attrs.ttf
                fontName = attrs.get("name", "")
                if not fontName:
                    self.error("You have to specify the TTF font name for %s" % ttf)
                else:
                    #if self.debug:
                    #    print "ttf", fontName
                    if fontName in self._fonts.keys():
                        self.warning("Repeated font embed for %s" % fontName)
                    else:
                        #~ print ttf, "FONTNAME: " + name
                        #if attrs["print"]:
                        #    self.add("FONTNAME: " + fontName)
                        self._registerFont(fontName, fontAlias)
                        font = TTFont(fontName, ttf)
                        #~ print font
                        pdfmetrics.registerFont(font)
                        addMapping(fontName, 0, 0, fontName) #normal
                        addMapping(fontName, 0, 1, fontName) #italic
                        addMapping(fontName, 1, 0, fontName) #bold
                        addMapping(fontName, 1, 1, fontName) #italic and bold

            elif attrs.afm is not None and attrs.pfb is not None:
                afm = attrs.afm
                pfb = attrs.pfb

                #~ print afm, pfb

                fontName = ""
                f = open(afm)
                found = 0
                while not found:
                    line = f.readline()[:-1]
                    if not found and line[:16] == 'StartCharMetrics':
                        raise "fontname not found"
                    if line[:8] == 'FontName':
                        fontName = line[9:]
                        found = 1
                f.close()

                #~ fontName = attrs.get("name", fontName.strip())
                fontName = fontName.strip()
                # print fontName

                givenFontName = attrs.get("name", "")
                if givenFontName and fontName !=givenFontName:
                    self.warning("The font name you specified is not equal with the real fontname: %s" % fontName)
                if not givenFontName:
                    self.warning("The font name %s for %s and %s is not specified" % (fontName, afm, pfm))

                #if attrs["print"]:
                #    self.add("FONTNAME: " + fontName)
                if fontName in self._fonts.keys():
                    self.warning("Repeated font embed for %s" % fontName)
                else:
                    self._registerFont(fontName, givenFontName + "," + fontAlias)
                    justFace = pdfmetrics.EmbeddedType1Face(afm, pfb)
                    pdfmetrics.registerTypeFace(justFace)
                    justFont = pdfmetrics.Font(fontName, fontName, attrs.encoding)
                    pdfmetrics.registerFont(justFont)

            else:
                self.error("wrong attributes for <fontembed>")

        except Exception, e:
            self.warning(e)
