# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################
from reportlab.platypus.flowables import KeepInFrame

__version__ = "$Revision: 1.14 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/04/05 13:39:06 $"

from pml_utils import *
from reportlab.platypus.tables import *
from reportlab.platypus.flowables import *
import copy

class PmlTable(Table):

    #def _listCellGeom(self, V, w, s, W=None, H=None, aH=72000):
    #    " This is a patch for buggy vAlign=top "
    #   return Table._listCellGeom(self, V, w, s, W, H, aH=72000)

    def _normWidth(self, w, maxw):
        " Helper for calculating percentages "
        if type(w)==type(""):
            w = ((maxw/100.0) * float(w[:-1]))
        elif (w is None) or (w=="*"):
            w = maxw
        return min(w, maxw)

    def wrap(self, availWidth, availHeight):

        # Strange bug, sometime the totalWidth is not set !?
        try:
            self.totalWidth
        except:
            self.totalWidth = availWidth

        # Prepare values
        totalWidth = self._normWidth(self.totalWidth, availWidth)
        remainingWidth = totalWidth
        remainingCols = 0
        newColWidths = self._colWidths

        # Calculate widths that are fix
        # IMPORTANT!!! We can not substitute the private value
        # self._colWidths therefore we have to modify list in place
        for i in range(len(newColWidths)):
            colWidth = newColWidths[i]
            if (colWidth is not None) or (colWidth=='*'):
                colWidth = self._normWidth(colWidth, totalWidth)
                remainingWidth -= colWidth
            else:
                remainingCols += 1
                colWidth = None
            newColWidths[i] = (colWidth)

        # Distribute remaining space
        if remainingCols:
            for i in range(len(newColWidths)):
                if newColWidths[i] is None:
                    newColWidths[i] = (remainingWidth / remainingCols) - 0.1

        # print "New values:", totalWidth, newColWidths, sum(newColWidths)

        # Call original method "wrap()"
        # self._colWidths = newColWidths
        return Table.wrap(self, availWidth, availHeight)

def _width(value=None):
    if value is None:
        return None
    value = str(value)
    if value.endswith("%"):
        return value
    return get_length(value)

class TableData:

    def __init__(self):
        self.data = []
        self.styles = []
        self.span = []

    def add_cell(self, data=None):
        self.col += 1
        self.data[len(self.data)-1].append(data)

    def add_style(self, data):
        self.styles.append(data)

    def add_empty(self, x, y):
        self.span.append((x, y))

    def get_data(self):
        data = self.data
        for x, y in self.span:
            try:
                data[y].insert(x, '')
            except:
                pass
        return data

class PDFTables:

    def start_table(self, attrs):
        self.next_para()

        tdata = TableData()

        # border
        tdata.border = attrs.border
        tdata.bordercolor = attrs.bordercolor

        # alignment
        #~ tdata.add_style(('VALIGN', (0,0), (-1,-1), attrs.valign.upper()))

        # Set Border and padding styles
        self._setCellStyle((0,0), (-1,-1), tdata, attrs, "table", default=attrs.cellpadding)

        # bgcolor
        #if attrs.bgcolor is not None:
        #    tdata.add_style(('BACKGROUND', (0, 0), (-1, -1), attrs.bgcolor))

        tdata.align = attrs.align.upper()
        tdata.col = 0
        tdata.row = 0
        tdata.colw = []
        tdata.rowh = []
        tdata.repeat = attrs.repeat
        tdata.width = _width(attrs.width)

        self.tabdata.append(tdata)

    def _TableData(self):
        return self.tabdata[-1]

    def end_table(self):
        tdata = self.tabdata.pop()
        data = tdata.get_data()

        #~ yy = 0
        #~ for y in data:
            #~ xx = 0
            #~ for x in y:
                #~ print "(%d,%d)" % (xx,yy), #repr(x),
                #~ xx += 1
            #~ yy += 1
            #~ print "."
        #~ for s in tdata.styles:
            #~ print "style", s

        #if self.debug:
        #    print "Data", repr(data)
        #    print "Width", tdata.colw
        #    print "Height", tdata.rowh

        #if self.debug:
        #    for s in tdata.styles:
        #        print repr(s)

        try:
            if tdata.data:
                t = PmlTable(
                    data,
                    colWidths = tdata.colw,
                    #colWidths = ["*", '33.3%', '33.3%'], #tdata.colw,
                    rowHeights = tdata.rowh,
                    # totalWidth = tdata.width,
                    splitByRow = 1,
                    # repeatCols = 1,
                    repeatRows = tdata.repeat,
                    hAlign = tdata.align,
                    vAlign = 'TOP',
                    #~ style = TableStyle([]))
                    style = TableStyle(tdata.styles))

                t.totalWidth = _width(tdata.width)
                # t.hAlign = tdata.align

                # print "table", t, tdata.colw, t.totalWidth
                self.add_story(t)

            else:
                self.warning("<table> is empty")
        except:
            self.warning("Error in <table>")
        self.next_para(None, "default")

    def start_tr(self, attrs):
        tdata = self._TableData()
        row = tdata.row
        begin = (0, row)
        end = (-1, row)

        #if attrs.bgcolor is not None:
        #    tdata.add_style(('BACKGROUND', begin, end, attrs.bgcolor))
        #if attrs.valign is not None:
        #    tdata.add_style(('VALIGN', begin, end, attrs.valign.upper()))

        # Set Border and padding styles
        self._setCellStyle(begin, end, tdata, attrs, "tr")
        
        #self._setBorderStyle("top", 'LINEABOVE', begin, end, tdata, attrs)
        #self._setBorderStyle("left", 'LINEBEFORE', begin, end, tdata, attrs)
        #self._setBorderStyle("right", 'LINEAFTER', begin, end, tdata, attrs)
        #self._setBorderStyle("bottom", 'LINEBELOW', begin, end, tdata, attrs)

        #if attrs.border:
        #    tdata.add_style(('GRID', begin, end, attrs.border, attrs.bordercolor))

        tdata.col = 0
        tdata.data.append([])

    def end_tr(self):
        tdata = self._TableData()
        tdata.row += 1

    def _getStyle(self, default, attrs, attrName, styleName, tagName=None):

        # Style attribute
        if attrs.has_key("style"):
            style = attrs.style.get(styleName, None)
            if style is not None:
                return style

        # Class attribute
        if attrs.has_key("class"):
            klass = attrs["class"]
            if klass:
                for k in klass.split():
                    style = self.styles.get(k, {}).get(styleName, None)
                    if style is not None:
                        return style

        # Attribute of tag
        if attrName and attrs.has_key(attrName):
            style = attrs[attrName]
            if style is not None:
                return style

        # Default style of tag
        if tagName:
            style = self.styles.get(tagName, {}).get(styleName, None)
            if style is not None:
                return style

        return default

    def _setBorderStyle(self, orientation, cmd, begin, end, tdata, attrs):
        orientation = orientation.lower().capitalize()
        styleb = self._getStyle(tdata.border, attrs, "border", "borderWidth" + orientation)
        stylec = self._getStyle(tdata.bordercolor, attrs, None, "borderColor" + orientation)
        styles = self._getStyle(tdata.bordercolor, attrs, None, "borderStyle" + orientation)
        if styleb:
            styles = str(styles).lower()
            if styles in ["butt", "round", "projecting", "squared"]:
                tdata.add_style((cmd, begin, end, styleb, stylec, styles))
            else:
                tdata.add_style((cmd, begin, end, styleb, stylec, 'squared'))
            return tdata
        return None

    def _setCellStyle(self, begin, end, tdata, attrs, mode=None, default=None):

        #LINECAPS={None: None, 'butt':0,'round':1,'projecting':2,'squared':2}
        #LINEJOINS={None: None, 'miter':0, 'mitre':0, 'round':1,'bevel':2}
        
        # Borders
        self._setBorderStyle("top", 'LINEABOVE', begin, end, tdata, attrs)
        self._setBorderStyle("left", 'LINEBEFORE', begin, end, tdata, attrs)
        self._setBorderStyle("right", 'LINEAFTER', begin, end, tdata, attrs)
        self._setBorderStyle("bottom", 'LINEBELOW', begin, end, tdata, attrs)
        
        # Paddings
        left = self._getStyle(default, attrs, "hpadding", "paddingLeft", mode)
        right = self._getStyle(default, attrs, "hpadding", "paddingRight", mode)
        top = self._getStyle(default, attrs, "vpadding", "paddingTop", mode)
        bottom = self._getStyle(default, attrs, "vpadding", "paddingBottom", mode)
        if left is not None:
            tdata.add_style(('LEFTPADDING', begin, end, left))
        if right is not None:
            tdata.add_style(('RIGHTPADDING', begin, end, right))
        if top is not None:
            tdata.add_style(('TOPPADDING', begin, end, top))
        if bottom is not None:
            tdata.add_style(('BOTTOMPADDING', begin, end, bottom))
        #if self.debug:
        #    print "style", mode, begin, end, left, right, top, bottom

        # Background
        bgcolor = self._getStyle(None, attrs, "bgcolor", "backColor", mode)        
        if bgcolor is not None:
            tdata.add_style(('BACKGROUND', begin, end, bgcolor))

        # Vertical Align
        if mode<>"table":
            valign = self._getStyle(None, attrs, "valign", "verticalAlign", mode)      
            if attrs.valign is not None:           
                tdata.add_style(('VALIGN', begin, end, valign.upper()))

    def _start_cell(self, attrs, mode="td"):
        tdata = self._TableData()

        cspan = attrs.colspan
        rspan = attrs.rowspan

        row = tdata.row
        col = tdata.col
        while 1:
            for x, y in tdata.span:
                if x==col and y==row:
                    col += 1
                    tdata.col += 1
            break
        cs = 0
        rs = 0

        begin = (col, row)
        end = (col, row)
        if cspan:
            end = (end[0] + cspan - 1, end[1])
        if rspan:
            end = (end[0], end[1] + rspan - 1)
        if begin!=end:
            #~ print begin, end
            tdata.add_style(('SPAN', begin, end))
            for x in range(begin[0], end[0]+1):
                for y in range(begin[1], end[1]+1):
                    if x!=begin[0] or y!=begin[1]:
                        tdata.add_empty(x, y)

        # Set Border and padding styles
        self._setCellStyle(begin, end, tdata, attrs, mode)

        #elif tdata.border:
        #    tdata.add_style(('BOX', begin, end, tdata.border, tdata.bordercolor))

        # Calculate widths

        # Add empty placeholders for new columns
        if col+1 > len(tdata.colw):
            tdata.colw = tdata.colw + ((col+1-len(tdata.colw)) * [_width()])
        # Get value of with, if no spanning
        if not cspan:
            width = self._getStyle(None, attrs, "width", "width", mode)
            # If is value, the set it in the right place in the arry
            if width is not None:
                tdata.colw[col] = _width(width)

        # Calculate heights
        if row+1 > len(tdata.rowh):
            tdata.rowh = tdata.rowh + ((row+1-len(tdata.rowh)) * [_width()])
        if not rspan:
            height = self._getStyle(None, attrs, "height", "height", mode)
            if height is not None:
                tdata.rowh[row] = _width(height)
                tdata.add_style(('FONTSIZE', begin, end, 1.0))
                tdata.add_style(('LEADING', begin, end, 1.0))
       
        #if self.debug:
        #    print mode, "pos", col, row, "w", width, "h", height
        
        """
        So wie das hier ist muss das auch fr andere Styles gemacht werden,
        die bei den Tabellenzellen verwendet werden sollen. Auerdem ist
        noch der via "class" bergebene Style und der tr-Style zu
        bercksichtigen !!!
        """

        #bgcolor = self._getStyle(None, attrs, "bgcolor", "backColor", mode)        
        #if bgcolor is not None:
        #    tdata.add_style(('BACKGROUND', begin, end, bgcolor))

        #~ tdata.add_style(('ALIGN', begin, end, 'CENTER'))

        #if attrs.valign is not None:
            #~ x = copy.deepcopy(end)
            #~ x[1] = x[1]+1
        #    tdata.add_style(('VALIGN', begin, end, attrs.valign.upper()))
            # print repr(('VALIGN', begin, end, attrs.valign.upper()))

        #if attrs.border:
        #    tdata.add_style(('GRID', begin, end, attrs.border, attrs.bordercolor))        

        # neue inhalte
        self.story.append([])
        self.next_para(attrs, mode)

    def start_td(self, attrs):
        self._start_cell(attrs, "td")

    def start_th(self, attrs):
        self._start_cell(attrs, "th")

    def end_td(self):
        tdata = self._TableData()

        # daten holen
        if not self.story[-1]:
            self.add_noop()
        self.next_para()

        # zellen hinzufgen
        cell = self.story.pop()
        tdata.add_cell(cell)

    end_th = end_td

    def start_keeptogether(self, attrs):
        self.story.append([])
        self.next_para()

    def end_keeptogether(self):
        if not self.story[-1]:
            self.add_noop()
        self.next_para()
        s = self.story.pop()
        self.add_story(KeepTogether(s))

    def start_keepinframe(self, attrs):
        self.story.append([])
        self.keepinframe = {
            "maxWidth": attrs["maxwidth"],
            "maxHeight": attrs["maxheight"],
            "mode": attrs["mode"],
            "name": attrs["name"],
            "mergeSpace": attrs["mergespace"]
            }
        # print self.keepinframe
        self.next_para()

    def end_keepinframe(self):
        if not self.story[-1]:
            self.add_noop()
        self.next_para()
        self.keepinframe["content"] = self.story.pop()
        self.add_story(KeepInFrame(**self.keepinframe))
