# -*- coding: ISO-8859-1 -*-
#############################################
## (C)opyright by Dirk Holtwick, 2002      ##
## All rights reserved                     ##
#############################################

__version__ = "$Revision: 1.5 $"
__author__  = "$Author: holtwick $"
__date__    = "$Date: 2007/01/23 11:13:33 $"

from reportlab.lib.units import inch, cm
from reportlab.lib.styles import *
from reportlab.lib.enums import *
from reportlab.lib.colors import *
from reportlab.lib.pagesizes import *
from reportlab.pdfbase import pdfmetrics
from reportlab.platypus import *
from reportlab.platypus.flowables import Flowable
from reportlab.platypus.para import Para, PageNumberObject, UNDERLINE, HotLink

import reportlab

# Version 2.0 of ReportLab uses Unicode! Test for version used
ReportlabVersion2 = (reportlab.Version >= "2.0")

import types

class PmlPara(Para):
    pass

    #~ def wrap(self, aw, ah):
        #~ result = Para.wrap(self, aw, ah)
        #~ #print "pml para wrap:", result
        #~ return result

def ErrorMsg(escape=0, short=0):
    """
    eine kleine funktion die auch fehlermeldungen zurckgibt,
    wenn es kein sys.stdout oder es andere probleme gibt.
    """

    import traceback, sys, cgi

    type=None
    value=None
    tb=None
    limit=None

    type, value, tb = sys.exc_info()
    list = traceback.format_tb(tb, limit) + traceback.format_exception_only(type, value)

    if short:
        body = list[-1]
    else:
        body = "Traceback (innermost last):\n" + "%-20s %s" % (
            string.join(list[:-1], ""), 
            list[-1], 
            )

    if escape: body = cgi.escape(body)

    return body

def toString(s, enc="latin1"):
    """
    Wrapper for all strings, for compatibility with ReportLab 2.0 
    """
    if ReportlabVersion2:
        # Convert to Unicode 
        if type(s) is not types.UnicodeType:
            s = unicode(str(s), enc, "ignore")
        # Euro sign helper
        s = s.replace(u"\x80", u"\u20ac")
        # s = s.encode("utf8")
    else:
        s = str(s)
    # Return normalized string
    return s

def toLength(s):
    """
    convert a string to  a length
    """
    try:
        s = str(s)
        # for us germans ;-)
        s = string.replace(s, ",", ".")
        if s[-2:]=='cm': return float(s[:-2])*cm
        if s[-2:]=='mm': return (float(s[:-2])*cm/10.0)
        if s[-2:]=='in': return float(s[:-2])*inch
        if s[-2:]=='pt': return float(s[:-2])
        if s[-2:]=='px': return float(s[:-2])
        if s[-1:]=='i': return float(s[:-1])*inch
        return float(s)
    except:
        raise ValueError, "Can't convert '%s' to length" % s

#########################################

get_length = toLength

def getColor(c):
    " Convert to color value "
    try:
        c = str(c).lower()
        if c.startswith("#") and len(c)==4:
            c = "#" + c[1] + c[1] + c[2] + c[2] + c[3] + c[3]
        return toColor(c)
    except:
        return None
    
# UNUSED
# def get_unicode(s):
#    return unicode(s, "iso-8859-1", 'ignore').encode("utf-8", 'ignore')

def getSize(s):
    """
    Converts strings to standard sizes
    Missing: em, ex, %
    """
    s = str(s).strip().lower().replace(",", ".")    
    if s[-2:]=='cm': 
        return float(s[:-2].strip()) * cm
    elif s[-2:]=='mm': 
        return (float(s[:-2].strip()) * cm / 10.0)
    elif s[-2:]=='in': 
        return float(s[:-2].strip()) * inch # 1pt == 1/72inch
    elif s[-2:]=='inch': 
        return float(s[:-4].strip()) * inch # 1pt == 1/72inch
    elif s[-2:]=='pt': 
        return float(s[:-2].strip())
    elif s[-2:]=='pc': 
        return float(s[:-2].strip()) * 12.0 # 1pt == 12pt
    elif s[-2:]=='px': 
        return float(s[:-2].strip())
    elif s[-1:]=='i':  # 1pt == 1/72inch
        return float(s[:-1].strip()) * inch
    return float(s)

def getCoords(x, y, w, h, pagesize):
    """
    As a stupid programmer I like to use the upper left
    corner of the document as the 0,0 coords therefore
    we need to do some fancy calculations
    """
    #~ print pagesize
    ax, ay = pagesize
    if x < 0:
        x = ax + x
    if y < 0:
        y = ay + y
    if w != None and h != None:
        if w < 0:
            w = (ax - x + w)
        if h < 0:
            h = (ay - y + h)
        return x, (ay - y - h), w, h
    return x, (ay - y)

def getBox(s, pagesize):
    """
    Parse sizes by corners in the form: 
    <X-Left> <Y-Upper> <Width> <Height>
    The last to values with negative values are interpreted as offsets form
    the right and lower border.
    """
    l = s.split()
    if len(l)<>4:
        raise Exception, "box not defined right way"
    x, y, w, h = map(getSize, l)
    return getCoords(x, y, w, h, pagesize)

def getPos(s, pagesize):
    """
    Pair of coordinates
    """
    l = string.split(s)
    if len(l)<>2:
        raise Exception, "position not defined right way"
    x, y = map(getSize, l)
    return getCoords(x, y, None, None, pagesize)

def getFontname(pool, name, default="helvetica"):
    """
    Name of a font
    """
    names = str(name).strip().lower().split(",")
    for name in names:
        font = pool.get(name.strip(), None)
        if font is not None:
            return font
    return pool.get(default, None)

# Style

from reportlab.lib.colors import white, black
from reportlab.lib.enums import TA_LEFT, TA_CENTER
from reportlab.lib.fonts import tt2ps

class PmlUnderLineHandler:

    def __init__(self, color=None):
        self.color = color

    def start_at(self, x, y, para, canvas, textobject):
        self.xStart = x
        self.yStart = y

    def end_at(self, x, y, para, canvas, textobject):
        if self.yStart==y:
            canvas.saveState()
            #print "underline", self.xStart, self.yStart, x, y, para.fontSize, para.leading
            offset = para.fontSize/8.0
            color = self.color
            if self.color is None:
                color = para.fontColor
            canvas.setStrokeColor(color)
            canvas.line(self.xStart, self.yStart-offset, x, y-offset)
            canvas.restoreState()

class PmlBackgroundHandler:

    def __init__(self, color=None):
        self.color = color

    def start_at(self, x, y, para, canvas, textobject):
        self.xStart = x
        self.yStart = y

    def end_at(self, x, y, para, canvas, textobject):
        if self.yStart==y:
            canvas.saveState()
            #print "background", self.xStart, self.yStart, x, y, para.fontSize, para.leading
            color = self.color
            if self.color is None:
                color = para.fontColor
            canvas.setFillColor(color)
            canvas.rect(
                self.xStart, 
                self.yStart + para.fontSize - para.leading, 
                x - self.xStart, 
                para.leading, 
                fill=1, stroke=0)
            canvas.restoreState()

class PmlLink:

    def __init__(self, url):
        self.url = url
        self.done = 0

    def start_at(self, x, y, para, canvas, textobject):
        self.xStart = x
        self.yStart = y

    def end_at(self, x, y, para, canvas, textobject):
        if not self.done:
            self.done = 1
            canvas.saveState()
            rect = [
                self.xStart, 
                self.yStart + para.fontSize - para.leading, 
                x, 
                self.yStart + para.fontSize]
            self.link(rect, canvas)
            canvas.restoreState()

    def link(self, rect, canvas):
        canvas.linkURL(self.url, rect, relative=1)

class PmlInternalLink(PmlLink):

    def link(self, rect, canvas):
        destinationname = self.url
        contents = ""
        canvas.linkRect(contents, destinationname, rect, Border="[0 0 0]")

class PmlAnchor(PmlLink):

    def link(self, rect, canvas):
        destinationname = self.url
        [x, y, x1, y1] = rect
        canvas.bookmarkHorizontal(destinationname, x, y1) # use the upper y

class PmlStyle:

    """
    This is a container for style informations. The real ReportLab
    Styles will be generated by the PmlStyleObj
    """

    ctr = 1

    def __init__(self, name=None, parent=None, preset={}):
        self._parent = parent
        if name is None:
            name = "tmp" + str(self.ctr)
            self.ctr += 1
        self._name = name
        self._data = {}
        for k, v in preset.items():
            self._data[k] = v

    def get_name(self):
        return self._name

    def get(self, name, default):
        try:
           return self._data[name]
        except KeyError, e:
           pass

        if self._parent is not None:
           try:
              return self._parent.get(name, default)
           except KeyError, e:
              pass

        return default

    def copy(self):
        return PmlStyle(self._name, self._parent, self._data)

    def __getitem__(self, name):
        return self._data.get(name, None)

    def __setitem__(self, name, value):
        self._data[name] = value

    def __repr__(self):
        return "<%s '%s'>" % (self.__class__.__name__, self._name)

    def collect(self):
        if self.__dict__["_parent"] is not None:
            data = {}
            data.update(self.__dict__["_parent"].collect())
            data.update(self._data)
            return data
        return self._data

    def dump(self, indent=''):
        print indent + 'name =', self._name
        print indent + 'parent =', self._parent
        data = self.collect()
        keylist = data.keys()
        keylist.sort()
        for key in keylist:
            value = data.get(key, None)
            print indent + '%s = %s' % (key, value)
        print


class PmlStyleObj:

    """
    An object that creates Reportlab Program Fragments for
    some defined styles. 
    """

    defaults = {
        'fontName': 'Times-Roman', 
        'fontSize': 10, 
        'leading': 12, 
        'leftIndent': 0, 
        'rightIndent': 0, 
        'firstLineIndent': 0, 
        'alignment': TA_LEFT, 
        'spaceBefore': 0, 
        'spaceAfter': 0, 
        'bulletFontName': 'Times-Roman', 
        'bulletFontSize': 10, 
        'bulletIndent': 8, 
        'textColor': black, 
        'backColor': None, 
        'underline': 0, 
        'underlineColor': None, 
        'bold': 0, 
        'italic': 0, 
        'sub': 0, 
        'super': 0, 
        }

    def __init__(self, name, parent=None, preset={}):
        self._parent = parent
        self._name = name
        self._cmd_start = []
        self._cmd_end = []
        for k, v in preset.items():
            self[k] = v

    def get_name(self):
        return self._name

    def get(self, name, default):
        return getattr(self, name)

    def has_key(self, name):
        return self.__dict__.has_key(name)

    def __getitem__(self, name):
        return getattr(self, name)

    def __setitem__(self, name, value):
        setattr(self, name, value)

    def __repr__(self):
        return "<%s '%s'>" % (self.__class__.__name__, self._name)

    def __getattr__(self, name):
        try:
           return self.__dict__[name]
        except KeyError, e:
           pass

        if self.__dict__["_parent"] is not None:
           try:
              return getattr(self.__dict__["_parent"], name)
           except KeyError, e:
              pass

        return self.defaults[name]

    def __str__(self):       
        return self._name

    def dump(self, indent=''):
        print indent + 'path =', self.find_path()
        print indent + 'name =', self._name
        print indent + 'parent =', self._parent
        keylist = self.defaults.keys()
        keylist.sort()
        for key in keylist:
            value = self.get(key, None)
            print indent + '%s = %s [%s]' % (key, value, self.find_style(key))
        print

    def find_path(self, path=""):
        path = "/" + self._name + path
        if self.__dict__["_parent"] is not None:
            path = self.__dict__["_parent"].find_path(path)
        return path

    def find_style(self, name):
        if self.__dict__.has_key(name):
            return self._name
        if self.__dict__["_parent"] is not None:
            return self.__dict__["_parent"].find_style(name)
        return "-"

    def add_operation(self, op):
        self._cmd_start.append(('lineOperation', op))
        self._cmd_end.append(('endLineOperation', op))

    def start_cmd(self, cmd):
        self._cmd_start.append(cmd)

    def end_cmd(self, cmd):
        self._cmd_end.append(cmd)

    def start_program(self, program, ref=None):
        if ref is None or self.alignment!=ref.alignment:
            program.append(("align", self.alignment))
        program.append(("push",))
        if ref is None or (self.fontName!=ref.fontName or self.bold!=ref.bold or self.italic!=ref.italic):
            try:
                #~ font = self.fontName
                font = tt2ps(self.fontName, self.bold, self.italic)
                program.append(("face", font))
                # print font
            except Exception, e:
                pass
        if ref is None or self.fontSize!=ref.fontSize:
            program.append(("size", self.fontSize))
        if ref is None or self.rightIndent!=ref.rightIndent:
            program.append(("rightIndent", self.rightIndent))
        if ref is None or self.textColor!=ref.textColor:
            program.append(("color", self.textColor))
        if ref is None or self.leading!=ref.leading:
            program.append(("leading", self.leading))
        if ref is None or self.underline!=ref.underline:
            if self.underline:
                self.add_operation(PmlUnderLineHandler(self.underlineColor))
        if ref is None or self.backColor!=ref.backColor:
            if self.backColor:
                self.add_operation(PmlBackgroundHandler(self.backColor))
        if ref is None or self["super"]!=ref["super"]:
            if self["super"]:
                size = self.fontSize * 0.7
                rise = self.fontSize * 0.5
                self.start_cmd(('size', size))
                self.start_cmd(('rise', rise))
                self.end_cmd(('size', self.fontSize))
                self.end_cmd(('rise', -rise))
        if ref is None or self["sub"]!=ref["sub"]:
            if self["sub"]:
                size = self.fontSize * 0.7
                rise = self.fontSize * 0.5
                self.start_cmd(('size', size))
                self.start_cmd(('rise', -rise))
                self.end_cmd(('size', self.fontSize))
                self.end_cmd(('rise', rise))
        if ref is None or self.leftIndent!=ref.leftIndent or self.firstLineIndent!=ref.firstLineIndent:
            program.append(("indent", self.leftIndent))
            if self.firstLineIndent:
                #print self.leftIndent, self.firstLineIndent, -1 * self.firstLineIndent
                program.append(("indent", self.firstLineIndent))
                #~ program.append( "xxx" )
                program.append(("indent", -1 * self.firstLineIndent))

        for cmd in self._cmd_start:
            program.append(cmd)
        self._cmd_start = []

    def stop_program(self, program):
        while len(self._cmd_end):
            program.append(self._cmd_end.pop())
        program.append(("pop",))

if __name__=="__main__":
    pNormal = PmlStyle('Normal', None)
    pNormal["fontName"] = 'Times-Roman'
    pNormal["fontSize"] = 12
    pNormal["leading"] = 14.4
    pNormal.dump()
    print
    pPre = PmlStyle('Literal', pNormal)
    pPre["fontName"] = 'Courier'
    pPre.dump()
    pNormal["test"] = 14.4
    pPre.dump()
